@@ -7,7 +7,7 @@ gptty: context-preserving chatGPT CLI wrapper
 __name__ = "gptty"
 __author__ = "Sig Janoska-Bedi"
 __credits__ = ["Sig Janoska-Bedi"]
-__version__ = "0.2.3"
+__version__ = "0.2.5"
 __license__ = "MIT"
 __maintainer__ = "Sig Janoska-Bedi"
 __email__ = "signe@atreeus.com"
@@ -18,6 +18,8 @@ import os
 import asyncio
 import pandas as pd
 import nltk
+import socket
+from contextlib import closing
 
 # app specific requirements
 from gptty.config import get_config_data
@@ -51,6 +53,24 @@ def return_log_as_df(configs):
         df = pd.DataFrame(columns=['timestamp','tag','question','response'])
     return df
 
+# Check if the system has a valid internet connection
+
+def has_internet_connection(host="google.com", port=443, timeout=3):
+    """Check if the system has a valid internet connection.
+
+    Args:
+        host (str, optional): A well-known website to test the connection. Default is 'www.google.com'.
+        port (int, optional): The port number to use for the connection. Default is 80.
+        timeout (int, optional): The time in seconds to wait for a response before giving up. Default is 3.
+
+    Returns:
+        bool: True if the system has an internet connection, False otherwise.
+    """
+    try:
+        with closing(socket.create_connection((host, port), timeout=timeout)):
+            return True
+    except OSError:
+        return False
 
 # borrowed version callback from https://click.palletsprojects.com/en/7.x/options/#callbacks-and-eager-options
 def print_version(ctx, param, value, version=__version__):
@@ -68,15 +88,16 @@ def main():
 
 @click.command()
 @click.option('--config_path', '-c', default=os.path.join(os.getcwd(),'gptty.ini'), help="Path to config file.")
-def chat(config_path):
+@click.option('--verbose', '-v', is_flag=True, help="Show debug data.")
+def chat(config_path:str, verbose:bool):
   
   """
   Run the gptty chat client
   """
 
-  asyncio.run(chat_async_wrapper(config_path))
+  asyncio.run(chat_async_wrapper(config_path, verbose))
 
-async def chat_async_wrapper(config_path):
+async def chat_async_wrapper(config_path:str, verbose:bool):
   title = r"""
                  _   _         
      ____  ____ | | | |        
@@ -89,46 +110,59 @@ async def chat_async_wrapper(config_path):
   """
 
   # Print the text in cyan
-  click.echo(f"{CYAN}{title}\nWelcome to gptty (v.{__version__}), a ChatGPT wrapper in your TTY.\nType :help in the chat interface if you need help getting started.{RESET}\n")
+  click.echo(f"{CYAN}{title}\nWelcome to gptty (v.{__version__}), a ChatGPT wrapper in your TTY.\nType :help in the chat interface if you need help getting started.{' Verbose / Debug mode is on.' if verbose else ''}{RESET}\n")
   
   if not os.path.exists(config_path):
-      click.echo(f"{RED}FAILED to access app config file at {config_path}. Are you sure this is a valid config file? Run `gptty chat --help` for more information.")
+      click.echo(f"{RED}FAILED to access app config file at {config_path}. Are you sure this is a valid config file? Run `gptty chat --help` for more information. You can get a sample config at <https://github.com/signebedi/gptty/blob/master/assets/gptty.ini.example>.")
       return
 
   # load the app configs
   configs = get_config_data(config_file=config_path)
-  
+
+  # Here, we verify that we have a wifi connection and if not, exit
+  if not has_internet_connection(configs['verify_internet_endpoint']):
+    click.echo(f"{RED}FAILED to verify connection at {configs['verify_internet_endpoint']}. Are you sure you are connected to the internet?")
+    return
+
   # create the output file if it doesn't exist
   with open (configs['output_file'], 'a'): pass
   
   # Run the main function
   # create_chat_room(configs=configs, config_path=config_path)
   # asyncio.run(create_chat_room(configs=configs, config_path=config_path))
-  await create_chat_room(configs=configs, config_path=config_path)
+  await create_chat_room(configs=configs, config_path=config_path, verbose=verbose)
 
 
 @click.command()
 # @click.option('--log', '-l', is_flag=True, callback=print_version,
 #               expose_value=False, is_eager=True, help="Show question log.")
 @click.option('--config_path', '-c', default=os.path.join(os.getcwd(),'gptty.ini'), help="Path to config file.")
+@click.option('--additional_context', '-a', default="", help="Pass more context to your questions.")
 @click.option('--question', '-q', multiple=True, help='Repeatable list of questions.')
 @click.option('--tag', '-t', default="", help='Tag to categorize your query. [optional]')
-def query(config_path, question, tag):
+@click.option('--verbose', '-v', is_flag=True, help="Show debug data.")
+@click.option('--json', '-j', is_flag=True, help="Return query as JSON object.")
+@click.option('--quiet', is_flag=True, help="Don't write to stdout.")
+def query(config_path:str, additional_context:str, question:str, tag:str, verbose:bool, json:bool, quiet:bool):
   """
   Submit a gptty query
   """
 
-  asyncio.run(query_async_wrapper(config_path, question, tag))
+  asyncio.run(query_async_wrapper(config_path, question, tag, additional_context, verbose, json, quiet))
 
 
-async def query_async_wrapper(config_path, question, tag):
+async def query_async_wrapper(config_path:str, question:str, tag:str, additional_context:str, verbose:bool, json:bool, quiet:bool):
   # load the app configs
   configs = get_config_data(config_file=config_path)
-  
+
+  # Here, we verify that we have a wifi connection and if not, exit
+  if not has_internet_connection(configs['verify_internet_endpoint']):
+    click.echo(f"{RED}FAILED to verify connection at {configs['verify_internet_endpoint']}. Are you sure you are connected to the internet?")
+    return
   # create the output file if it doesn't exist
   with open (configs['output_file'], 'a'): pass
 
-  await run_query(questions=question, tag=tag, configs=configs, config_path=config_path)
+  await run_query(questions=question, tag=tag, configs=configs, additional_context=additional_context, config_path=config_path, verbose=verbose, return_json=json, quiet=quiet)
 
 
 @click.command()
@@ -154,4 +188,4 @@ main.add_command(query)
 main.add_command(log)
 
 if __name__ == "__main__":
-  main()
\ No newline at end of file
+  main()
