@@ -1,20 +1,23 @@
 __name__ = "gptty.gptty"
 __author__ = "Sig Janoska-Bedi"
 __credits__ = ["Sig Janoska-Bedi"]
-__version__ = "0.2.3"
+__version__ = "0.2.5"
 __license__ = "MIT"
 __maintainer__ = "Sig Janoska-Bedi"
 __email__ = "signe@atreeus.com"
 
-import openai
-import time
-from datetime import datetime
 import click
+import openai
 import pandas as pd
-import os
-import sys
-import asyncio
 from aioconsole import ainput
+from datetime import datetime
+import os, time, sys, asyncio, json
+
+# prompt toolkit requirements
+from prompt_toolkit import PromptSession
+from prompt_toolkit.formatted_text import ANSI
+from prompt_toolkit.styles import Style
+from prompt_toolkit.patch_stdout import patch_stdout
 
 # app specific requirements
 from gptty.tagging import get_tag_from_text
@@ -25,6 +28,10 @@ from gptty.config import get_config_data
 CYAN = "\033[1;36m"
 RED = "\033[1;31m"
 RESET = "\033[0m"
+YELLOW = "\033[1;33m"
+GREY = "\033[90m"
+ERASE_LINE = "\033[K"
+MOVE_CURSOR_UP = "\033[F"
 
 HELP = """
                         [Commands]
@@ -104,7 +111,7 @@ async def wait_graphic():
             print("\b" * 10, end="", flush=True)
 
 # this is used when we run the `chat` command
-async def create_chat_room(configs=get_config_data(), log_responses=True, config_path=None):
+async def create_chat_room(configs=get_config_data(), log_responses:bool=True, config_path=None, verbose:bool=False):
 
     # Authenticate with OpenAI using your API key
     # click.echo (configs['api_key'])
@@ -121,6 +128,7 @@ async def create_chat_room(configs=get_config_data(), log_responses=True, config
         df = pd.DataFrame(columns=['timestamp','tag','question','response'])
 
     try:
+
         openai.api_key = configs['api_key'].rstrip('\n')
     except:
         click.echo(f"{RED}FAILED to initialize connection to OpenAI. Have you added an API token? See gptty docs <https://github.com/signebedi/gptty#configuration> or <https://platform.openai.com/account/api-keys> for more information.")
@@ -138,15 +146,27 @@ async def create_chat_room(configs=get_config_data(), log_responses=True, config
         click.echo(f"{RED}FAILED to validate the model name '{model_engine}'. Are you sure this is a valid OpenAI model? Check the available models at <https://platform.openai.com/docs/models/overview> and try again.{RESET}")
         return
 
+    session = PromptSession()
+
     # Continuously send and receive messages
     while True:
-        # Get user input
-        i = await ainput(f"{CYAN}> ")
-        tag,question = get_tag_from_text(i)
-        prompt_length = len(question)
 
-        if prompt_length < 1:
-            click.echo('\nPlease provide an actual prompt.\n')
+        # Get user input
+        try:
+
+            with patch_stdout():
+                i = await session.prompt_async(ANSI(f"{CYAN}> "), style=Style.from_dict({'': 'ansicyan'}))
+            print(f"{ERASE_LINE}{MOVE_CURSOR_UP}{GREY}> {i}\n", end="")
+
+            # i = await ainput(f"{CYAN}> ")
+            tag,question = get_tag_from_text(i)
+            prompt_length = len(question)
+
+        # handle keyboard interrupt
+        except KeyboardInterrupt:
+            i = False
+
+        if i == False:
             continue
         elif i.strip() in [':help',':h']:
             click.echo(HELP)
@@ -162,18 +182,17 @@ async def create_chat_room(configs=get_config_data(), log_responses=True, config
             # c = f'{"|".join(f"{row[]}" for index,row in df.iterrows())}'.replace('|','\n')
             click.echo (f'\n{df}\n')
             continue
-        elif i.strip().startswith(':'):
-            click.echo('\nPlease provide a valid command.\n')
+        elif i.strip().startswith(':') or prompt_length < 1:
+            click.echo('\nPlease provide a valid command or prompt.\n')
             continue
 
-
         # click.echo the question in color
-        print(f"{CYAN}[{configs['your_name']}] {question}{RESET} \n", end="", flush=True)
+        print(f"\n{CYAN}[{configs['your_name']}] {question}{RESET} \n", end="", flush=True)
 
         # we create the callable wait_graphic task
         wait_task = asyncio.create_task(wait_graphic())
 
-        fully_contextualized_question = get_context(tag, configs['max_context_length'],configs['output_file'],context_keywords_only=configs['context_keywords_only'], model_type=model_type, question=question)
+        fully_contextualized_question = get_context(tag, configs['max_context_length'], configs['output_file'], model_engine, context_keywords_only=configs['context_keywords_only'], model_type=model_type, question=question, debug=verbose)
 
         response_task = asyncio.create_task(fetch_response(fully_contextualized_question, model_engine, max_tokens, temperature, model_type))
 
@@ -209,7 +228,7 @@ async def create_chat_room(configs=get_config_data(), log_responses=True, config
 
 
 # this is used when we run the `query` command
-async def run_query(questions:list, tag:str, configs=get_config_data(), log_responses=True, config_path=None):
+async def run_query(questions:list, tag:str, configs=get_config_data(), additional_context:str="", log_responses:bool=True, config_path=None, verbose:bool=False, return_json:bool=False, quiet:bool=False):
 
     if not os.path.exists(config_path):
         click.echo(f"{RED}FAILED to access app config file at {config_path}. Are you sure this is a valid config file? Run `gptty chat --help` for more information.")
@@ -239,6 +258,9 @@ async def run_query(questions:list, tag:str, configs=get_config_data(), log_resp
         click.echo(f"{RED}FAILED to initialize connection to OpenAI. Have you added an API token? See gptty docs <https://github.com/signebedi/gptty#configuration> or <https://platform.openai.com/account/api-keys> for more information.")
         return
 
+    # create a json representation of the question / response data FOR THE CURRENT SESSION
+    json_output = []
+
     # Set the parameters for the OpenAI completion API
     model_engine = configs['model'].rstrip('\n')
     temperature = configs['temperature'] # controls the creativity of the response
@@ -255,35 +277,50 @@ async def run_query(questions:list, tag:str, configs=get_config_data(), log_resp
         if len(question) < 1:
             continue
 
+        if not return_json and not quiet:
+            # click.echo the question in color
+            print(f"{CYAN}[{configs['your_name']}] {question}{RESET} \n", end="", flush=True)
 
-        # click.echo the question in color
-        print(f"{CYAN}[{configs['your_name']}] {question}{RESET} \n", end="", flush=True)
+            # we create the callable wait_graphic task
+            wait_task = asyncio.create_task(wait_graphic())
 
-
-        # we create the callable wait_graphic task
-        wait_task = asyncio.create_task(wait_graphic())
-
-        fully_contextualized_question = get_context(tag, configs['max_context_length'],configs['output_file'],context_keywords_only=configs['context_keywords_only'], model_type=model_type, question=question)
+        fully_contextualized_question = get_context(tag, configs['max_context_length'], configs['output_file'], model_engine, additional_context=additional_context, context_keywords_only=configs['context_keywords_only'], model_type=model_type, question=question, debug=verbose)
 
         response_task = asyncio.create_task(fetch_response(fully_contextualized_question, model_engine, max_tokens, temperature, model_type))
 
         # Wait for the response to be completed
         response = await response_task
 
-        # Cancel the wait graphic task
-        wait_task.cancel()
-        print("\b" * 10 , end="", flush=True)
+        if not return_json and not quiet:
+            # Cancel the wait graphic task
+            wait_task.cancel()
+            print("\b" * 10 , end="", flush=True)
 
-        response_text = response.choices[0].text.strip() if model_type == 'v1/completions' else response.choices[0]['messages']['content'].strip()
-        deformatted_response_text = response.choices[0].text.strip().replace("\n", " ") if model_type == 'v1/completions' else response.choices[0]['messages']['content'].strip().replace("\n", " ")
+        response_text = response.choices[0].text.strip() if model_type == 'v1/completions' else response.choices[0]['message']['content'].strip()
+        deformatted_response_text = response.choices[0].text.strip().replace("\n", " ") if model_type == 'v1/completions' else response.choices[0]['message']['content'].strip().replace("\n", " ")
 
         if configs['preserve_new_lines']:
-            click.echo(f"\b{RED}[{configs['gpt_name']}] {response_text}{RESET}\n")
+            response_text_to_print = f"\b{RED}[{configs['gpt_name']}] {response_text}{RESET}\n"
         else:
             # click.echo the response in color
-            click.echo(f"\b{RED}[{configs['gpt_name']}] {deformatted_response_text}{RESET}\n")
+            response_text_to_print = f"\b{RED}[{configs['gpt_name']}] {deformatted_response_text}{RESET}\n"
 
         if log_responses:
             timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
             with open (configs['output_file'], 'a') as f:
                 f.write(f"{timestamp}|{tag}|{question.replace('|','')}|{deformatted_response_text.replace('|','')}\n")
+
+        if return_json or quiet:
+            json_output.append({
+                'question': question,
+                'response': deformatted_response_text
+            })
+        else:
+            click.echo(response_text_to_print)
+
+    # Add this line before the final return statement
+    if return_json and not quiet:
+        json_response = json.dumps(json_output)
+        click.echo(json_response)
+        # return json_response
+        return
\ No newline at end of file
